"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


import os
import time
from tempfile import TemporaryDirectory

from git import Repo

from pagure_exporter.conf import standard
from pagure_exporter.view.misc import tnfsprog, tnfswarn


class PushRepo:
    def __init__(self):
        self.srce = standard.srcehuto
        self.dest = standard.desthuto
        self.pkey = standard.pagucode
        self.gkey = standard.gtlbcode
        self.brcs = standard.brtocopy
        self.sarg = dict(prefix=standard.prfxsrce, dir=standard.tempdrct)
        self.darg = dict(prefix=standard.prfxdest, dir=standard.tempdrct)
        self.sloc = TemporaryDirectory(**self.sarg)
        self.dloc = TemporaryDirectory(**self.darg)
        standard.srcecloc = self.sloc.name
        standard.destcloc = self.dloc.name

    def downsrce(self):
        try:
            strttime = time.time()
            Repo.clone_from(url=self.srce, to_path=self.sloc.name)
            stoptime = time.time()
            return True, "%.2f" % (stoptime - strttime)
        except Exception as expt:
            return False, str(expt)

    def downdest(self):
        try:
            strttime = time.time()
            Repo.clone_from(url=self.dest, to_path=self.dloc.name)
            stoptime = time.time()
            return True, "%.2f" % (stoptime - strttime)
        except Exception as expt:
            return False, str(expt)

    def cbrcsrce(self):
        if os.path.exists(os.path.join(self.sloc.name, ".git")):
            repoobjc = Repo(path=self.sloc.name)
            brcslist = [
                refx.name.replace("%s/" % standard.dfremote, "")
                for refx in repoobjc.remote(standard.dfremote).refs
            ]
            standard.sbrcavbl = list(brcslist)
            return True, brcslist
        else:
            return False, "Cloned source namespace assets could not be found"

    def cbrcdest(self):
        if os.path.exists(os.path.join(self.dloc.name, ".git")):
            repoobjc = Repo(path=self.dloc.name)
            brcslist = [
                refx.name.replace("%s/" % standard.dfremote, "")
                for refx in repoobjc.remote(standard.dfremote).refs
            ]
            standard.dbrcavbl = list(brcslist)
            return True, brcslist
        else:
            return False, "Cloned destination namespace assets could not be found"

    def tnfsrepo(self):
        if os.path.exists(os.path.join(self.sloc.name, ".git")):
            strttime = time.time()
            repoobjc = Repo(path=self.sloc.name)
            repoobjc.create_remote(standard.nrmtname, url=standard.desthuto)
            if len(standard.brtocopy) == 0:
                standard.tnfsqant = len(standard.sbrcavbl)
                tnfswarn(False, standard.tnfsqant)
                for brdx in standard.sbrcavbl:
                    repoobjc.git.checkout("%s" % brdx)
                    repoobjc.git.push(standard.nrmtname, "--set-upstream", brdx, "--force")
                    tnfsprog(
                        brdx,
                        standard.sbrcavbl.index(brdx) + 1,
                        len(standard.sbrcavbl),
                        True,
                    )
                    standard.tnfsindx += 1
            else:
                standard.tnfsqant = len(standard.brtocopy)
                tnfswarn(True, standard.tnfsqant)
                for brdx in standard.brtocopy:
                    if brdx in standard.sbrcavbl:
                        repoobjc.git.checkout("%s" % brdx)
                        repoobjc.git.push(standard.nrmtname, "--set-upstream", brdx, "--force")
                        tnfsprog(
                            brdx,
                            standard.brtocopy.index(brdx) + 1,
                            len(standard.brtocopy),
                            True,
                        )
                        standard.tnfsindx += 1
                    else:
                        tnfsprog(
                            brdx,
                            standard.brtocopy.index(brdx) + 1,
                            len(standard.brtocopy),
                            False,
                        )
            stoptime = time.time()
            return True, "%.2f" % (stoptime - strttime)
        else:
            return False, "Cloned namespace assets could not be found"



"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


import time
from datetime import datetime

import requests

from pagure_exporter.conf import standard


class MoveTkts:
    def __init__(self):
        self.purl = f"{standard.pagulink}/{standard.srcename}"
        self.gurl = f"{standard.gtlblink}/{standard.destname}"
        self.phed = {"Authorization": f"token {standard.pagucode}"}
        self.ghed = {"Authorization": f"Bearer {standard.gtlbcode}"}

    def getcount(self):
        try:
            strttime = time.time()
            initdata = {"per_page": standard.pagesize, "page": "1"}
            if standard.tktstate == "closed" or standard.tktstate == "all":
                initdata["status"] = standard.tktstate
            initresp = requests.get(
                url=f"{self.purl}/issues",
                params=initdata,
                headers=self.phed,
                timeout=standard.rqsttime,
            )
            respcode, respresn = initresp.status_code, initresp.reason
            if respcode == 200:
                initdict = initresp.json()
                if initdict["pagination"]["next"] is None:
                    standard.tktcount = initdict["total_issues"]
                else:
                    standard.pageqant = int(initdict["pagination"]["pages"])
                    lastdata = {
                        "per_page": standard.pagesize,
                        "page": standard.pageqant,
                    }
                    if standard.tktstate == "closed" or standard.tktstate == "all":
                        lastdata["status"] = standard.tktstate
                    lastresp = requests.get(
                        url=f"{self.purl}/issues",
                        params=lastdata,
                        headers=self.phed,
                        timeout=standard.rqsttime,
                    )
                    respcode, respresn = lastresp.status_code, lastresp.reason
                    if lastresp.status_code == 200:
                        lastdict = lastresp.json()
                        lastqant = int(lastdict["total_issues"])
                        standard.tktcount = (standard.pageqant - 1) * standard.pagesize + lastqant
            stoptime = time.time()
            timereqd = "%.2f" % (stoptime - strttime)
            return respcode, respresn, timereqd
        except Exception as expt:
            return False, expt, "0"

    def iterpage(self, indx):
        try:
            strttime = time.time()
            rqstdata = {"per_page": standard.pagesize, "page": f"{indx}"}
            if standard.tktstate == "closed" or standard.tktstate == "all":
                rqstdata["status"] = standard.tktstate
            response = requests.get(
                url=f"{self.purl}/issues",
                params=rqstdata,
                headers=self.phed,
                timeout=standard.rqsttime,
            )
            respcode, respresn = response.status_code, response.reason
            if respcode == 200:
                standard.pagerslt = response.json()["issues"]
            stoptime = time.time()
            timereqd = "%.2f" % (stoptime - strttime)
            return respcode, respresn, timereqd
        except Exception as expt:
            return False, expt, "0"

    def iteriden(self, tkid):
        try:
            strttime = time.time()
            response = requests.get(
                url=f"{self.purl}/issue/{tkid}",
                headers=self.phed,
                timeout=standard.rqsttime,
            )
            respcode, issuskip = response.status_code, True
            if respcode == 200:
                standard.issurslt = response.json()
                # If the preferred ticket status is "FULL", all statuses must be addressed
                if (
                    standard.issurslt["status"].lower() == standard.tktstate == "closed"
                    or standard.issurslt["status"].lower() == standard.tktstate == "open"
                    or standard.tktstate == "all"
                ):
                    issuskip = False
            else:
                issuskip = response.reason
            stoptime = time.time()
            timereqd = "%.2f" % (stoptime - strttime)
            return respcode, issuskip, timereqd
        except Exception as expt:
            return False, expt, "0"

    def itertkts(self, dictobjc):
        try:
            strttime = time.time()
            standard.rateindx += 1
            standard.issuname = dictobjc["title"]
            standard.issuiden = dictobjc["id"]
            standard.isclosed = True if dictobjc["status"] == "Closed" else False
            standard.authname = dictobjc["user"]["fullname"]
            standard.authlink = dictobjc["user"]["full_url"]
            standard.authorid = dictobjc["user"]["name"]
            standard.issulink = dictobjc["full_url"]
            standard.issutags = dictobjc["tags"]
            standard.issubody = dictobjc["content"]
            standard.issecret = dictobjc["private"]
            standard.timedata = int(dictobjc["date_created"])
            headdata = standard.headtemp_ticket.format(
                issuiden=standard.issuiden,
                issuname=standard.issuname,
                timeout=standard.rqsttime,
            )
            bodydata = standard.bodytemp_ticket.format(
                issubody=standard.issubody,
                issulink=standard.issulink,
                reponame=standard.srcename,
                repolink=standard.srcedict["repolink"],
                authname=standard.authname,
                authlink=standard.authlink,
                dateinfo=datetime.utcfromtimestamp(standard.timedata).strftime("%c"),
                mo=datetime.utcfromtimestamp(standard.timedata).strftime("%b").lower(),
                dd=datetime.utcfromtimestamp(standard.timedata).strftime("%d"),
                yy=datetime.utcfromtimestamp(standard.timedata).strftime("%Y"),
                hh=datetime.utcfromtimestamp(standard.timedata).strftime("%H"),
                mm=datetime.utcfromtimestamp(standard.timedata).strftime("%M"),
            )
            """
            Replace "@" in the `bodydata` with "&" to ensure that wrong people are not referenced
            in the destination namespace
            Check https://github.com/gridhead/pagure-exporter/issues/7 for more information about
            the problem
            """
            rqstdata = {"title": headdata, "description": bodydata.replace("@", "&")}
            if standard.movetags:
                rqstdata["labels"] = ",".join(standard.issutags)
            if standard.movehush:
                rqstdata["confidential"] = standard.issecret
            response = requests.post(
                url=f"{self.gurl}/issues",
                data=rqstdata,
                headers=self.ghed,
                timeout=standard.rqsttime,
            )
            respcode, respresn = response.status_code, response.reason
            if respcode == 201:
                respresn = response.json()["web_url"]
                standard.gtlbtkid = response.json()["iid"]
                standard.issutnfs += 1
            stoptime = time.time()
            timereqd = "%.2f" % (stoptime - strttime)
            return respcode, respresn, timereqd
        except Exception as expt:
            return False, expt, "0"

    def itercmts(self, dictobjc):
        try:
            strttime = time.time()
            standard.rateindx += 1
            standard.cmtsiden = dictobjc["id"]
            standard.cmtslink = f"{standard.issulink}#comment-{standard.cmtsiden}"
            standard.cmtsauth = dictobjc["user"]["fullname"]
            standard.cmtsaurl = dictobjc["user"]["full_url"]
            standard.cmtstime = int(dictobjc["date_created"])
            standard.cmtsbody = dictobjc["comment"]
            bodydata = standard.bodytemp_cmts.format(
                cmtsbody=standard.cmtsbody,
                cmtslink=standard.cmtslink,
                cmtsauth=standard.cmtsauth,
                cmtsaurl=standard.cmtsaurl,
                issulink=standard.issulink,
                reponame=standard.srcename,
                repolink=standard.srcedict["repolink"],
                dateinfo=datetime.utcfromtimestamp(standard.cmtstime).strftime("%c"),
                mo=datetime.utcfromtimestamp(standard.cmtstime).strftime("%b").lower(),
                dd=datetime.utcfromtimestamp(standard.cmtstime).strftime("%d"),
                yy=datetime.utcfromtimestamp(standard.cmtstime).strftime("%Y"),
                hh=datetime.utcfromtimestamp(standard.cmtstime).strftime("%H"),
                mm=datetime.utcfromtimestamp(standard.cmtstime).strftime("%M"),
            )
            """
            Replace "@" in the `bodydata` with "&" to ensure that wrong people are not referenced
            in the destination namespace
            Check https://github.com/gridhead/pagure-exporter/issues/7 for more information about
            the problem
            """
            rqstdata = {"body": bodydata.replace("@", "&")}
            response = requests.post(
                url=f"{self.gurl}/issues/{standard.gtlbtkid}/notes",
                data=rqstdata,
                headers=self.ghed,
                timeout=standard.rqsttime,
            )
            respcode, respresn = response.status_code, response.reason
            if respcode == 201:
                respresn = f"{standard.destdict['repolink']}/-/issues/{standard.gtlbtkid}#note_{response.json()['id']}"  # noqa: E501
                standard.cmtsqant += 1
            stoptime = time.time()
            timereqd = "%.2f" % (stoptime - strttime)
            return respcode, respresn, timereqd
        except Exception as expt:
            return False, expt, "0"

    def iterstat(self):
        try:
            strttime, respcode, respresn = time.time(), 0, ""
            standard.rateindx += 1
            if standard.isclosed:
                rqstdata = {"state_event": "close"}
                response = requests.put(
                    url=f"{self.gurl}/issues/{standard.gtlbtkid}",
                    data=rqstdata,
                    headers=self.ghed,
                    timeout=standard.rqsttime,
                )
                respcode, respresn = response.status_code, response.reason
            else:
                respcode, respresn = 0, "0"
            stoptime = time.time()
            timereqd = "%.2f" % (stoptime - strttime)
            return respcode, respresn, timereqd
        except Exception as expt:
            return False, expt, "0"



"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


from pagure_exporter.view.dcrt import general, warning


def tnfsprog(brchname, indx, qant, avbl):
    if avbl:
        general(
            "[%d/%d] Branch '%s' was transferred to the destination namespace"
            % (int(indx), int(qant), str(brchname))
        )
    else:
        general(
            "[%d/%d] Branch '%s' was not found in the source namespace"
            % (int(indx), int(qant), str(brchname))
        )


def tnfswarn(avbl, qant):
    if avbl:
        warning("Transferring %d requested branches" % int(qant))
    else:
        warning("Transferring %d available branches" % int(qant))



"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


from logging import getLogger
from logging.config import dictConfig

"""
STANDARD CONFIGURATION VARIABLES
"""

movecmts = False

movetags = False

tktgroup = []

movestat = False

movehush = False

# While the location for creating temporary directories is definitive, the temporary directories
# are created with a random name constituting of a definitive prefix in runtime with only the user
# that created the file having READ and WRITE access to them. Basically, while the location
# `/var/tmp` is known from the beginning, the locations `/var/tmp/pexp-tempsrce-$RANDOM` and
# `/var/tmp/pexp-tempdest-$RANDOM` are not and hence - that should not be a security concern.
# For more information, please read
# https://bandit.readthedocs.io/en/latest/plugins/b108_hardcoded_tmp_directory.html
# https://security.openstack.org/guidelines/dg_using-temporary-files-securely.html
tempdrct = "/var/tmp"  # noqa: S108

# If a definitive timeout is not specified for every usage of `requests`, there is likeliness that
# a failure in probing a URL might lead to the program trying so indefinitely - thus causing the
# program to freeze. Remember, `requests` is a synchronous HTTP library and if you are looking for
# some of that nice asynchronicity, you are much better off using the likes of `httpx`.
# For more information, please read
# https://bandit.readthedocs.io/en/latest/plugins/b113_request_without_timeout.html
# https://datagy.io/python-requests-timeouts/
rqsttime = 60

dfremote = "origin"
prfxsrce = "pexp-tempsrce-"
prfxdest = "pexp-tempdest-"
frgesrce = "pagure.io"
frgedest = "gitlab.com"
nrmtname = "freshsrc"

"""
INTERNAL CONFIGURATION VARIABLES
"""

paguuser = "UNAVAILABLE"
pagucode = "UNAVAILABLE"
srcename = "UNAVAILABLE"
pagulink = "https://pagure.io/api/0"
srcedict = {
    "makedate": "0",
    "lastmode": "0",
    "descript": "UNAVAILABLE",
    "repolink": "UNAVAILABLE",
    "reponame": "UNAVAILABLE",
    "identity": "UNAVAILABLE",
    "tagslist": [],
    "maintain": {
        "username": "UNAVAILABLE",
        "fullname": "UNAVAILABLE",
    },
}
srcehuto = "UNAVAILABLE"
srcedisp = "UNAVAILABLE"

gtlbuser = "UNAVAILABLE"
gtlbcode = "UNAVAILABLE"
destname = "UNAVAILABLE"
gtlblink = "https://gitlab.com/api/v4/projects"
destdict = {
    "makedate": "0",
    "lastmode": "0",
    "descript": "UNAVAILABLE",
    "repolink": "UNAVAILABLE",
    "reponame": "UNAVAILABLE",
    "identity": "UNAVAILABLE",
    "tagslist": [],
    "maintain": {
        "username": "UNAVAILABLE",
        "fullname": "UNAVAILABLE",
    },
}
desthuto = "UNAVAILABLE"
destdisp = "UNAVAILABLE"

brtocopy = []
sbrcavbl = []
dbrcavbl = []
srcecloc = "UNAVAILABLE"
destcloc = "UNAVAILABLE"

rateindx = 0

# Time in seconds to wait for when the rate limit for API requests is reached
waittime = 60

# Rate limit for API requests as defined by the destination forge
# See https://github.com/fedora-infra/pagure-exporter/issues/60#issuecomment-1820304156
ratebond = 195

# Current count of branches on the namespaces to be transferred
tnfsindx = 0

# Total quantity of branches on the namespaces to be transferred
tnfsqant = 0

# Number of issue ticket entities to be present on a response page of issues
pagesize = 35

# State of the issue ticket entities to be considered for transferring
tktstate = "open"

# Quantity of issue tickets available on the source namespace
tktcount = 0

# Current page number for probing into the available issue tickets in the source namespace
pageqant = 1

# List of issue ticket entities present on a response page of issues
pagerslt = []

# Dictionary of information pertaining to a single issue ticket made in the source namespace
issurslt = {}

# Title information of the issue ticket made in the source namespace
issuname = "UNAVAILABLE"

# Identity of the issue ticket made in the source namespace
issuiden = "0"

# Current status of the issue ticket made in the source namespace
isclosed = None

# Hyperlink of the issue ticket made in the source namespace
issulink = "UNAVAILABLE"

# Labels associated with the issue ticket made in the source namespace
issutags = []

# Textual information of the issue ticket made in the source namespace
issubody = "UNAVAILABLE"

# Privacy information of the issue ticket made in the source namespace
issecret = "UNAVAILABLE"

# Username of the issue ticket made in the source namespace
authorid = "UNAVAILABLE"

# Full name of the issue ticket made in the source namespace
authname = "UNAVAILABLE"

# Fedora Account System URL for the author of the issue ticket made in the source namespace
authlink = "UNAVAILABLE"

# Creation datetime information of the issue ticket made in the source namespace
timedata = 0

# Identity of the issue ticket transferred to the destination namespace
gtlbtkid = "UNAVAILABLE"

# Quantity of issue tickets transferred to the destination namespace
issutnfs = 0

# Template for the title information of the issue tickets to be transferred
headtemp_ticket = "[SN#{issuiden}] {issuname}"

# Template for the textual information of the issue tickets to be transferred
bodytemp_ticket = """
{issubody}

_This issue ticket was originally created [here]({issulink}) on a Pagure repository,
[**{reponame}**]({repolink}) by [**{authname}**]({authlink}) on
[**{dateinfo}** UTC](https://savvytime.com/converter/utc/{mo}-{dd}-{yy}/{hh}-{mm})._

_This issue ticket was automatically created by the
[**Pagure Exporter**](https://github.com/gridhead/pagure-exporter)._
"""

# List of comment entities made under a ticket in the source namespace
issucmts = []

# Identity for the comments made under a ticket in the source namespace
cmtsiden = "UNAVAILABLE"

# Hyperlink for the comments made under a ticket in the source namespace
cmtslink = "UNAVAILABLE"

# Fullname of the author of the comments made under a ticket in the source namespace
cmtsauth = "UNAVAILABLE"

# Fedora Account System URL for the author of the comments made under a ticket in the source
# namespace
cmtsaurl = "UNAVAILABLE"

# Creation datetime information of the comments made under a ticket in the source namespace
cmtstime = 0

# Textual information of the comments made under a ticket in the source namespace
cmtsbody = "UNAVAILABLE"

# Hyperlink for the comments made under a transferred ticket in the destination namespace
gtlbcurl = "UNAVAILABLE"

# Quantity of comments made under a ticket in the source namespace
cmtsqant = 0

# Template for the textual information of the comments to be transferred
bodytemp_cmts = """
{cmtsbody}

_This comment was originally created [here]({cmtslink}) by [**{cmtsauth}**]({cmtsaurl}) under
[this]({issulink}) issue ticket on a Pagure repository, [**{reponame}**]({repolink}) on
[**{dateinfo}** UTC](https://savvytime.com/converter/utc/{mo}-{dd}-{yy}/{hh}-{mm})._

_This comment was automatically created by the
[**Pagure Exporter**](https://github.com/gridhead/pagure-exporter)._
"""

# The default configuration for service logging
logrconf = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "standard": {
            "format": "%(asctime)s %(message)s",
            "datefmt": "[%Y-%m-%d %I:%M:%S %z]",
        },
    },
    "handlers": {
        "console": {
            "level": "INFO",
            "formatter": "standard",
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",
        },
    },
    "root": {
        "level": "INFO",
        "handlers": ["console"],
    },
}

dictConfig(logrconf)

logger = getLogger(__name__)



"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


import sys

from pagure_exporter.conf import standard
from pagure_exporter.view.dcrt import failure, general, section, success, warning
from pagure_exporter.work.repo import PushRepo


def showrepo():
    try:
        section("Starting migration...")
        pushobjc = PushRepo()
        section("Attempting source namespace assets clone...")
        sclorslt = pushobjc.downsrce()
        if sclorslt[0]:
            success("Source namespace assets clone succeeded!")
            general("Directory: %s" % str(standard.srcecloc))
            general("Time taken: %s second(s)" % str(sclorslt[1]))
            section("Attempting destination namespace assets clone...")
            dclorslt = pushobjc.downdest()
            if dclorslt[0]:
                success("Destination namespace assets clone succeeded!")
                general("Directory: %s" % str(standard.destcloc))
                general("Time taken: %s second(s)" % str(dclorslt[1]))
                section("Reading branches data from the locally cloned assets...")
                sbrcrslt, dbrcrslt = pushobjc.cbrcsrce(), pushobjc.cbrcdest()
                if sbrcrslt[0] and dbrcrslt[0]:
                    success("Branches data reading succeeded!")
                    general("Available in source namespace: %d branch(es)" % len(standard.sbrcavbl))
                    for indx in standard.sbrcavbl:
                        general("  - (SRCE branch) %s" % str(indx))
                    general(
                        "Available in destination namespace: %d branch(es)" % len(standard.dbrcavbl)
                    )
                    for indx in standard.dbrcavbl:
                        general("  - (DEST branch) %s" % str(indx))
                    general("Requested for transferring: %d branch(es)" % len(standard.brtocopy))
                    for indx in standard.brtocopy:
                        general("  - (RQST branch) %s" % str(indx))
                    section("Initializing namespace assets transfer...")
                    tnfsrslt = pushobjc.tnfsrepo()
                    general(
                        "Assets transferred: %d branch(es) completed, %d branch(es) requested"
                        % (int(standard.tnfsindx), int(standard.tnfsqant))
                    )
                    general("Time taken: %s second(s)" % str(tnfsrslt[1]))
                    if tnfsrslt[0]:
                        if standard.tnfsindx == standard.tnfsqant:
                            success("Namespace assets transfer succeeded!")
                            sys.exit(0)
                        elif 0 < standard.tnfsindx < standard.tnfsqant:
                            warning("Namespace assets transfer partially completed!")
                            sys.exit(2)
                        else:  # pragma: no cover
                            # Tested already in `test_unit_tnfsrepo`
                            # From `test/test_unit_repo`
                            failure("Namespace assets transfer failed!")
                            sys.exit(1)
                    else:  # pragma: no cover
                        # Tested already in `test_unit_tnfsrepo`
                        # From `test/test_unit_repo`
                        failure("Namespace assets transfer failed!")
                        general("Exception occurred: %s" % str(tnfsrslt[1]))
                        sys.exit(1)
                else:  # pragma: no cover
                    # Tested already in `test_unit_cbrcsrce` and `test_unit_cbrcdest`
                    # From `test/test_unit_repo`
                    failure("Branches data reading failed!")
                    erormesg = str(sbrcrslt[1]) if not sbrcrslt[0] else str(dbrcrslt[1])
                    general("Exception occurred: %s" % erormesg)
                    sys.exit(1)
            else:  # pragma: no cover
                # Tested already in `test_unit_downdest`
                # From `test/test_unit_repo`
                failure("Destination namespace assets clone failed!")
                general("Exception occurred: %s" % str(dclorslt[1]))
                sys.exit(1)
        else:  # pragma: no cover
            # Tested already in `test_unit_downsrce`
            # From `test/test_unit_repo`
            failure("Source namespace assets clone failed!")
            general("Exception occurred: %s" % str(sclorslt[1]))
            sys.exit(1)
    except Exception as expt:
        failure("Migration failed!")
        general("Exception occurred: %s" % str(expt))
        sys.exit(1)



"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


import click

from pagure_exporter import __version__ as versobjc
from pagure_exporter.view.repo import showrepo
from pagure_exporter.view.stat import showstat
from pagure_exporter.view.tkts import showtkts
from pagure_exporter.work.keep import keepbrcs, keeptkts, storeinf


@click.group(name="pagure_exporter")
@click.option(
    "-s",
    "--srce",
    "srce",
    required=True,
    help="Source namespace for importing assets from",
)
@click.option(
    "-d",
    "--dest",
    "dest",
    required=True,
    help="Destination namespace for exporting assets to",
)
@click.option(
    "-p",
    "--pkey",
    "pkey",
    required=True,
    help="Pagure API key for accessing the source namespace",
)
@click.option(
    "-g",
    "--gkey",
    "gkey",
    required=True,
    help="GitLab API key for accessing the destination namespace",
)
@click.option(
    "-f",
    "--fusr",
    "fusr",
    required=True,
    help="Username of the account that owns the Pagure API key",
)
@click.option(
    "-t",
    "--tusr",
    "tusr",
    required=True,
    help="Username of the account that owns the GitLab API key",
)
@click.version_option(
    version=versobjc,
    prog_name=click.style(
        "Pagure Exporter by Akashdeep Dhar <t0xic0der@fedoraproject.org>",
        fg="green",
        bold=True,
    ),
)
def main(srce, dest, pkey, gkey, fusr, tusr):
    """
    Pagure Exporter
    """
    storeinf(srce, dest, pkey, gkey, fusr, tusr)


@main.command(
    name="tkts",
    help="Initiate transfer of issue tickets",
    context_settings={"show_default": True},
)
@click.option(
    "-s",
    "--status",
    "status",
    type=click.Choice(["OPEN", "SHUT", "FULL"], case_sensitive=False),
    help="Extract issue tickets of the mentioned status",
    default="OPEN",
)
@click.option(
    "-r",
    "--ranges",
    "ranges",
    nargs=2,
    help="Extract issue tickets in the mentioned ranges",
    default=None,
)
@click.option(
    "-p",
    "--select",
    "select",
    type=str,
    help="Extract issue tickets of the selected numbers",
    default=None,
)
@click.option(
    "-c",
    "--comments",
    "comments",
    help="Transfer all the associated comments",
    default=False,
    is_flag=True,
)
@click.option(
    "-l",
    "--labels",
    "labels",
    help="Migrate all the associated labels",
    default=False,
    is_flag=True,
)
@click.option(
    "-a",
    "--commit",
    "commit",
    help="Assert issue ticket states as they were",
    default=False,
    is_flag=True,
)
@click.option(
    "-t",
    "--secret",
    "secret",
    help="Confirm issue ticket privacy as they were",
    default=False,
    is_flag=True,
)
def main_transfer_tkts(status, select, ranges, comments, labels, commit, secret):
    if select is not None and ranges is not None:
        raise click.UsageError("The `select` and `ranges` options cannot be used together")

    tktgroup = []

    if select is not None:
        try:
            tktgroup = [int(indx.strip()) for indx in select.split(",")]
        except Exception as expt:
            raise click.BadParameter(
                message="The provided parameters for the `select` option could not be parsed"
            ) from expt

    if ranges is not None:
        try:
            ranges = [int(indx) for indx in ranges]
            tktgroup = [indx for indx in range(min(ranges), max(ranges) + 1)]
        except Exception as expt:
            raise click.BadParameter(
                message="The provided parameters for the `ranges` option could not be parsed"
            ) from expt

    keeptkts(status, tktgroup, comments, labels, commit, secret)
    showstat()
    showtkts()


@main.command(
    name="repo",
    help="Initialize transfer of repository assets",
    context_settings={"show_default": True},
)
@click.option("-b", "--brcs", "brcs", type=str, default=None, help="List of branches to extract")
def main_transfer_repo(brcs):
    brcslist = []

    if brcs is not None:
        brcslist = [indx.strip() for indx in brcs.split(",")]

    keepbrcs(brcslist)
    showstat()
    showrepo()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pagure_exporter',
 'pagure_exporter.conf',
 'pagure_exporter.view',
 'pagure_exporter.work']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.0,<4.0.0', 'click>=8.1.3,<9.0.0', 'requests>=2.28.0,<3.0.0']

entry_points = \
{'console_scripts': ['pagure-exporter = pagure_exporter.main:main']}

setup_kwargs = {
    'name': 'pagure-exporter',
    'version': '0.1.2',
    'description': 'Simple exporter tool that helps migrate repository files, data assets and issue tickets from projects on Pagure to GitLab',
    'long_description': "# Pagure Exporter\n\nSimple exporter tool that helps migrate repository files, data assets and issue tickets from projects on Pagure to GitLab\n\n## Installation\n\n### From PyPI\n\n1. Ensure that you have `python3`, `virtualenv` and `python3-pip` installed.\n\n    ```\n    $ sudo dnf install python3 virtualenv python3-pip --setopt=install_weak_deps=False\n    ```\n\n2. Create and activate a Python virtual environment in that directory.\n\n    ```\n    $ virtualenv venv\n    ```\n\n    ```\n    (venv) $ source venv/bin/activate\n    ```\n\n    Sample output\n\n    ```\n    created virtual environment CPython3.11.5.final.0-64 in 143ms\n      creator CPython3Posix(dest=/home/archdesk/Projects/pagure-exporter/venv, clear=False, no_vcs_ignore=False, global=False)\n      seeder FromAppData(download=False, pip=bundle, setuptools=bundle, wheel=bundle, via=copy, app_data_dir=/home/archdesk/.local/share/virtualenv)\n        added seed packages: pip==23.2.1, setuptools==68.0.0, wheel==0.41.1\n      activators BashActivator,CShellActivator,FishActivator,NushellActivator,PowerShellActivator,PythonActivator\n    ```\n\n3. Install `pagure-exporter` using `pip` in the activated virtual environment.\n\n    ```\n    (venv) $ pip install pagure-exporter\n    ```\n\n    Sample output\n\n    ```\n    Requirement already satisfied: pagure-exporter in ./venv/lib/python3.12/site-packages (0.1.0)\n    Requirement already satisfied: GitPython<4.0.0,>=3.1.37 in ./venv/lib/python3.12/site-packages (from pagure-exporter) (3.1.40)\n    Requirement already satisfied: click<9.0.0,>=8.1.3 in ./venv/lib/python3.12/site-packages (from pagure-exporter) (8.1.7)\n    Requirement already satisfied: requests<3.0.0,>=2.31.0 in ./venv/lib/python3.12/site-packages (from pagure-exporter) (2.31.0)\n    Requirement already satisfied: tqdm<5.0.0,>=4.64.1 in ./venv/lib/python3.12/site-packages (from pagure-exporter) (4.66.1)\n    Requirement already satisfied: gitdb<5,>=4.0.1 in ./venv/lib/python3.12/site-packages (from GitPython<4.0.0,>=3.1.37->pagure-exporter) (4.0.10)\n    Requirement already satisfied: charset-normalizer<4,>=2 in ./venv/lib64/python3.12/site-packages (from requests<3.0.0,>=2.31.0->pagure-exporter) (3.3.0)\n    Requirement already satisfied: idna<4,>=2.5 in ./venv/lib/python3.12/site-packages (from requests<3.0.0,>=2.31.0->pagure-exporter) (3.4)\n    Requirement already satisfied: urllib3<3,>=1.21.1 in ./venv/lib/python3.12/site-packages (from requests<3.0.0,>=2.31.0->pagure-exporter) (2.0.7)\n    Requirement already satisfied: certifi>=2017.4.17 in ./venv/lib/python3.12/site-packages (from requests<3.0.0,>=2.31.0->pagure-exporter) (2023.7.22)\n    Requirement already satisfied: smmap<6,>=3.0.1 in ./venv/lib/python3.12/site-packages (from gitdb<5,>=4.0.1->GitPython<4.0.0,>=3.1.37->pagure-exporter) (5.0.1)\n    ```\n\n### From source\n\n1. Ensure that you have `git`, `python3`, `virtualenv` and `poetry` installed.\n\n    ```\n    $ sudo dnf install git python3 virtualenv poetry --setopt=install_weak_deps=False\n    ```\n\n2. Clone the repository to the local storage and make it the present working directory.\n\n    ```\n    $ git clone https://github.com/gridhead/pagure-exporter.git\n    ```\n\n    ```\n    $ cd pagure-exporter\n    ```\n\n    Sample output\n\n    ```\n    Cloning into 'pagure-exporter'...\n    remote: Enumerating objects: 118, done.\n    remote: Counting objects: 100% (118/118), done.\n    remote: Compressing objects: 100% (78/78), done.\n    remote: Total 118 (delta 48), reused 94 (delta 31), pack-reused 0\n    Receiving objects: 100% (118/118), 56.38 KiB | 4.34 MiB/s, done.\n    Resolving deltas: 100% (48/48), done.\n    ```\n\n3. Create and activate a Python virtual environment in that directory.\n\n    ```\n    $ virtualenv venv\n    ```\n\n    ```\n    (venv) $ source venv/bin/activate\n    ```\n\n    Sample output\n\n    ```\n    created virtual environment CPython3.11.5.final.0-64 in 143ms\n      creator CPython3Posix(dest=/home/archdesk/Projects/pagure-exporter/venv, clear=False, no_vcs_ignore=False, global=False)\n      seeder FromAppData(download=False, pip=bundle, setuptools=bundle, wheel=bundle, via=copy, app_data_dir=/home/archdesk/.local/share/virtualenv)\n        added seed packages: pip==23.2.1, setuptools==68.0.0, wheel==0.41.1\n      activators BashActivator,CShellActivator,FishActivator,NushellActivator,PowerShellActivator,PythonActivator\n    ```\n\n4. Check the project configuration's validity and then install the project dependencies.\n\n    ```\n    (venv) $ poetry check\n    ```\n\n    ```\n    (venv) $ poetry install\n    ```\n\n    Sample output\n\n    ```\n    All set!\n    ```\n\n    ```\n    Installing dependencies from lock file\n\n    Package operations: 19 installs, 0 updates, 0 removals\n\n      • Installing smmap (5.0.0)\n      • Installing certifi (2022.12.7)\n      • Installing charset-normalizer (3.0.1)\n      • Installing click (8.1.3)\n      • Installing gitdb (4.0.10)\n      • Installing idna (3.4)\n      • Installing mccabe (0.6.1)\n      • Installing mypy-extensions (0.4.3)\n      • Installing pathspec (0.11.0)\n      • Installing platformdirs (2.6.2)\n      • Installing pycodestyle (2.8.0)\n      • Installing pyflakes (2.4.0)\n      • Installing urllib3 (1.26.14)\n      • Installing black (22.12.0)\n      • Installing flake8 (4.0.1)\n      • Installing gitpython (3.1.30)\n      • Installing isort (5.12.0)\n      • Installing requests (2.28.2)\n      • Installing tqdm (4.64.1)\n\n    Installing the current project: pagure-exporter (0.1.0)\n    ```\n\n## Usage\n\n### Setup\n\n1. Using an internet browser of your choice, open up [**Pagure**](https://pagure.io) and login to your account.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/01.png)\n\n2. Click on your profile display picture and then, head over to the [**Account Settings**](https://pagure.io/settings) page.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/02.png)\n\n3. Under the [**API Keys**](https://pagure.io/settings#nav-api-tab) section, click on the [**Create new API key**](https://pagure.io/settings/token/new) button in the top right corner.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/03.png)\n\n4. As this is the source namespace, check all the ACLs that are required to read the asset information associated with a repository, set a safe expiration date for the API token and write an appropriate description for its usage before clicking on the [**Create**](#) button.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/04.png)\n\n5. Make note of the API token generated and ensure that they are not shared with others or used for a different purpose.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/05.png)\n\n6. Head over to a repository that the currently logged-in username has at least a READ access to.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/06.png)\n\n7. Make note of the source namespace in the format of `HOLDER/REPONAME` where the `HOLDER` can be a group or a sole user.\n   For example, in case of a repository located at `https://pagure.io/fedora-infra/ansible` - the source namespace would be `fedora-infra/ansible`.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/07.png)\n\n8. In another internet browser tab or window, open up [**GitLab**](https://gitlab.com/) and login to your account.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/08.png)\n\n9. Click on the [**New project/repository**](https://gitlab.com/projects/new) option from the sidebar of the profile page and then, click on the [**Create blank project**](https://gitlab.com/projects/new#blank_project) option.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/09.png)\n\n10. Create a new empty repository which will act as the destination for the asset transfer. It is recommended to have the same name as the source namespace to avoid confusion, but it is not strictly required.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/10.png)\n\n11. Head over to the created repository and make note of the **Project ID**.\n    For example in this case, it is `42823949` for the destination repository named `gridhead/pagure-exporter-test`.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/11.png)\n\n12. Expand the sidebar to head over to the [**Repository**](#) section from the [**Settings**](#) section.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/17.png)\n\n13. In the [**Protected Branches**](#) section, turn off all existing *branch protection rules* and *allow force pushing* to all branches temporarily to allow for the repository assets to be moved in here.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/18.png)\n\n14. Expand the sidebar to head over to the [**Access Tokens**](#) section from the [**Settings**](#) section.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/12.png)\n\n15. In the [**Project Access Token**](#) page, click on the [**Add new token**](#) button to begin creating a new access token.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/13.png)\n\n16. As this is the destination namespace, check all the scopes that are required to write the asset information associated with a repository, pick an appropriate role, set a safe expiration date and write an appropriate description for its usage before clicking on the [**Create project access token**](#) button.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/14.png)\n\n17. Make note of the API token generated and ensure that they are not shared with others or used for a different purpose.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/15.png)\n\n18. Ensure that you have the following information handy before proceeding to the next steps.\n\n    1. Username of an account that has at least the READ permissions in the source namespace on Pagure (Say `srceuser`)\n    2. Access token belonging to the aforementioned account with appropriate ACLs checked required for at least the READ permissions in the source namespace (Say `srcecode`)\n    3. Name of the source namespace in the format `HOLDER/REPONAME` where the `HOLDER` can be a group or a sole user (Say `srcerepo`)\n    4. Username of an account that has at least the WRITE permissions in the destination namespace on GitLab (Say `destuser`)\n    5. Access token belonging to that aforementioned account appropriate roles and scopes required for at least the WRITE permissions in the destination namespace (Say `destcode`)\n    6. Name of the destination namespace in the format of uniquely identifiable `PROJECTID` string (Say `destrepo`)\n\n## Operate\n\n### View help menu\n\n1. Check the current version of the installed project as well as the usage information.\n\n    ```\n    (venv) $ pagure-exporter --version\n    ```\n\n    ```\n    (venv) $ pagure-exporter --help\n    ```\n\n    Sample output\n\n    ```\n    Pagure Exporter by Akashdeep Dhar <t0xic0der@fedoraproject.org>, version 0.1.0\n    ```\n\n    ```\n    Usage: pagure-exporter [OPTIONS] COMMAND [ARGS]...\n\n    Options:\n      -s, --srce TEXT  Source namespace for importing assets from  [required]\n      -d, --dest TEXT  Destination namespace for exporting assets to  [required]\n      -p, --pkey TEXT  Pagure API key for accessing the source namespace\n                       [required]\n      -g, --gkey TEXT  GitLab API key for accessing the destination namespace\n                       [required]\n      -f, --fusr TEXT  Username of the account that owns the Pagure API key\n                       [required]\n      -t, --tusr TEXT  Username of the account that owns the GitLab API key\n                       [required]\n      --version        Show the version and exit.\n      --help           Show this message and exit.\n\n    Commands:\n      repo  Initialize transfer of repository assets\n      tkts  Initiate transfer of issue tickets\n    ```\n\n2. Check the usage information of the available subcommands.\n\n    ```\n    (venv) $ pagure-exporter -s a -d a -p a -g a -f a -t a repo --help\n    ```\n\n    ```\n    (venv) $ pagure-exporter -s a -d a -p a -g a -f a -t a tkts --help\n    ```\n\n    Sample output\n\n    ```\n    Usage: pagure-exporter repo [OPTIONS]\n\n      Initialize transfer of repository assets\n\n    Options:\n      -b, --brcs TEXT  List of branches to extract\n      --help           Show this message and exit.\n    ```\n\n    ```\n    Usage: pagure-exporter tkts [OPTIONS]\n\n      Initiate transfer of issue tickets\n\n    Options:\n      -s, --status [OPEN|SHUT|FULL]  Extract issue tickets of the mentioned status\n                                     [default: OPEN]\n      -r, --ranges TEXT...           Extract issue tickets in the mentioned ranges\n      -p, --select TEXT              Extract issue tickets of the selected numbers\n      -c, --comments                 Transfer all the associated comments\n      -l, --labels                   Migrate all the associated labels\n      -a, --commit                   Assert issue ticket states as they were\n      -t, --secret                   Confirm issue ticket privacy as they were\n      --help                         Show this message and exit.\n    ```\n\n### Migrate repository files\n\n1. Ensure that the location where the project repository was cloned is the present working directory and that the previously populated virtual environment is enabled.\n\n    ```\n    $ cd pagure-exporter\n    ```\n\n    ```\n    $ source venv/bin/activate\n    ```\n\n2. Using an internet browser of your choice, visit the source namespace repository page on Pagure to pick the branches that you wish to transfer.\n\n    ![](https://raw.githubusercontent.com/gridhead/pagure-exporter/main/data/16.png)\n\n3. Execute the following command to begin migrating the repository assets from the source namespace on Pagure to the destination namespace on GitLab.\n\n    1. If only a set of branches are to be migrated\n\n        ```\n        (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 repo \\\n                 --brcs brca,brcb,brcc,brcd\n        ```\n\n        For a set of branches available in the source namespace named `brca`, `brcb`, `brcc` and `brcd` to be migrated to the destination namespace.\n\n    2. If all the available branches are to be migrated\n\n        ```\n        (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 repo\n        ```\n\n       This is the default behaviour of the subcommand so if no branch names are provided, all the branches from the source namespace are migrated\n\n4. If there are any contents available in the branches of the destination namespace, they will be overwritten by the contents from the branches migrated from the source namespace.\n\n### Migrate issue tickets\n\n1. Ensure that the location where the project repository was cloned is the present working directory and that the previously populated virtual environment is enabled.\n\n    ```\n    $ cd pagure-exporter\n    ```\n\n    ```\n    $ source venv/bin/activate\n    ```\n\n2. Execute the following command to begin extracting the issue tickets from the source namespace on Pagure to the destination namespace.\n\n   1. If the issue tickets of a certain status need to be transferred.\n\n      ```\n      (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 tkts \\\n                 --status open\n      ```\n\n   2. If the comments associated with the issue tickets need to be transferred.\n\n      ```\n      (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 tkts \\\n                 --comments\n      ```\n\n   3. If the labels associated with the issue tickets need to be transferred.\n\n      ```\n      (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 tkts \\\n                 --labels\n      ```\n\n   4. If the states associated with the issue tickets need to be transferred.\n\n      ```\n      (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 tkts \\\n                 --commit\n      ```\n\n   5. If the privacy associated with the issue tickets need to be transferred.\n\n      ```\n      (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 tkts \\\n                 --secret\n      ```\n\n   6. If the issue tickets from a range of issue identities need to be transferred.\n\n      ```\n      (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 tkts \\\n                 --ranges STRT STOP\n      ```\n\n      Issue tickets with identities `STRT`, `STRT+1` ... `STOP-1`, `STOP` would be considered here.\n\n   7. If the issue tickets that need to be considered need to be cherry-picked.\n\n      ```\n      (venv) $ pagure-exporter \\\n                 --fusr srceuser --pkey srcecode --srce srcerepo \\\n                 --tusr destuser --gkey destcode --dest destrepo \\\n                 tkts \\\n                 --select NUM1,NUM2,NUM3 ...\n      ```\n\n      Issue tickets with identities `NUM1`, `NUM2`, `NUM3` ... would be considered here.\n\n   While these options can be mixed and matched to be used together, the options `--ranges` and `--select` cannot be used at the same time as they perform identical functions.\n\n   For example,\n\n      1. The following command will migrate all issue tickets, the identities of which fall between the range of `STRT` and `STOP` both included, with status `OPEN` along with the associated comments, labels and privacy.\n\n         ```\n         (venv) $ pagure-exporter \\\n                    --fusr srceuser --pkey srcecode --srce srcerepo \\\n                    --tusr destuser --gkey destcode --dest destrepo \\\n                    tkts \\\n                    --status open \\\n                    --comments \\\n                    --labels \\\n                    --secret \\\n                    --ranges STRT STOP\n         ```\n\n      2. The following command will migrate all issue tickets with the identities `NUM1`, `NUM2`, `NUM3` ... with status `SHUT` along with the associated labels, states and privacy.\n\n         ```\n         (venv) $ pagure-exporter \\\n                    --fusr srceuser --pkey srcecode --srce srcerepo \\\n                    --tusr destuser --gkey destcode --dest destrepo \\\n                    tkts \\\n                    --status shut \\\n                    --labels \\\n                    --commit \\\n                    --secret \\\n                    --select NUM1,NUM2,NUM3 ...\n         ```\n",
    'author': 'Akashdeep Dhar',
    'author_email': 'akashdeep.dhar@gmail.com',
    'maintainer': 'Akashdeep Dhar',
    'maintainer_email': 'akashdeep.dhar@gmail.com',
    'url': 'https://github.com/gridhead/pagure-exporter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)



"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


from pagure_exporter.conf import standard


def storeinf(srce, dest, pkey, gkey, fusr, tusr):
    standard.srcename = srce
    standard.destname = dest
    standard.pagucode = pkey
    standard.gtlbcode = gkey
    standard.paguuser = fusr
    standard.gtlbuser = tusr


def keepbrcs(brcs):
    standard.brtocopy = list(brcs)


def keeptkts(status, tktgroup, comments, labels, commit, secret):
    # Vote what kind of issue tickets are to be moved
    # Default Open
    if status == "SHUT":
        standard.tktstate = "closed"
    elif status == "FULL":
        standard.tktstate = "all"
    else:
        standard.tktstate = "open"

    # Set a list of ticket identities to be moved
    # Default Empty
    standard.tktgroup = tktgroup

    # Vote if comments associated with the issue tickets are to be moved
    # Default False
    standard.movecmts = comments

    # Vote if labels associated with the issue tickets are to be moved
    # Default False
    standard.movetags = labels

    # Vote if the state associated with the issue tickets are to be moved
    # Default False
    standard.movestat = commit

    # Vote if the privacy associated with the issue tickets are to be moved
    # Default False
    standard.movehush = secret



"""
Pagure Exporter
Copyright (C) 2022-2023 Akashdeep Dhar

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <https://www.gnu.org/licenses/>.

Any Red Hat trademarks that are incorporated in the source code or
documentation are not subject to the GNU General Public License and may only
be used or replicated with the express permission of Red Hat, Inc.
"""


import sys
import time

from pagure_exporter.conf import standard
from pagure_exporter.view.dcrt import failure, general, section, success, warning
from pagure_exporter.work.tkts import MoveTkts


def callwait():
    if standard.rateindx == standard.ratebond:
        warning(f"Rate limit reached - {standard.ratebond} API requests made...")
        general(
            f"Waiting for {standard.waittime} second(s) and resetting the counter before resuming the transfer process"  # noqa: E501
        )
        time.sleep(standard.waittime)
        standard.rateindx = 0


def showtkts():
    moveobjc = MoveTkts()
    section("Attempting source namespace issue ticket count...")
    warning(
        f"Extracting {'all ' if standard.tktstate == 'open' or standard.tktstate == 'closed' else ''}"  # noqa: E501
        f"{standard.tktstate} issue tickets {'with' if standard.movetags else 'without'} labels, "
        f"{'with' if standard.movestat else 'without'} states, "
        f"{'with' if standard.movehush else 'without'} privacy and "
        f"{'with' if standard.movecmts else 'without'} comments off the given selection"
    )
    if not standard.tktgroup:
        qantrslt = moveobjc.getcount()
        if qantrslt[0] == 200:
            general(
                f"Found {standard.tktcount} issue ticket(s) across {standard.pageqant} "
                f"page(s) in {qantrslt[2]} second(s)"
            )
            for indx in range(standard.pageqant):
                section(
                    f"Reading issue tickets information (Page {indx + 1} of {standard.pageqant})..."  # noqa: E501
                )
                pagerslt = moveobjc.iterpage(indx + 1)
                if pagerslt[0] == 200:
                    general(
                        f"Found {len(standard.pagerslt)} issue ticket(s) on this "
                        f"page in {pagerslt[2]} second(s)"
                    )
                    for jndx in standard.pagerslt:
                        callwait()
                        issurslt = moveobjc.itertkts(jndx)
                        section(
                            f"Migrating issue ticket {'with' if standard.movetags else 'without'} "  # noqa: E501
                            f"labels #{standard.issuiden} '{standard.issuname}' by "
                            f"'{standard.authname} (ID {standard.authorid})'..."
                        )
                        if issurslt[0] == 201:
                            general(f"Migrated to {issurslt[1]} in {issurslt[2]} second(s)")
                            if standard.movestat:
                                callwait()
                                section("Asserting issue ticket status...")
                                statrslt = moveobjc.iterstat()
                                if statrslt[0] == 200:
                                    general(
                                        f"Asserted CLOSE status of the ticket in {statrslt[2]} second(s)"  # noqa: E501
                                    )
                                elif statrslt[0] == 0:
                                    general(
                                        "Assertion unnecessary due to the OPEN status of the ticket"  # noqa: E501
                                    )
                                else:  # pragma: no cover
                                    # Tested already in `test_unit_itertkts`
                                    # From `test/test_unit_tkts`
                                    failure("Issue ticket status assertion failed!")
                                    general(
                                        f"Failed due to code '{statrslt[0]}' and reason '{statrslt[1]}' "  # noqa: E501
                                        f"in {statrslt[2]} second(s)"
                                    )
                            if standard.movecmts:
                                section("Reading comment information...")
                                standard.issucmts = jndx["comments"]
                                general(
                                    f"Found {len(standard.issucmts)} entities in 0.00 second(s)"
                                )
                                for kndx in standard.issucmts:
                                    callwait()
                                    cmtsrslt = moveobjc.itercmts(kndx)
                                    section(
                                        f"Transferring comment (Entity {standard.cmtsqant} of "
                                        f"{len(standard.issucmts)})..."
                                    )
                                    if cmtsrslt[0] == 201:
                                        general(
                                            f"Transferred to {cmtsrslt[1]} in {cmtsrslt[2]} second(s)"  # noqa: E501
                                        )
                                    else:  # pragma: no cover
                                        # Tested already in `test_unit_itercmts`
                                        # From `test/test_unit_tkts`
                                        failure("Comment transfer failed!")
                                        general(
                                            f"Failed due to code '{cmtsrslt[0]}' and reason '{cmtsrslt[1]}' "  # noqa: E501
                                            f"in {cmtsrslt[2]} second(s)"
                                        )
                                        sys.exit(1)
                                standard.cmtsqant = 0
                        else:  # pragma: no cover
                            # Tested already in `test_unit_itertkts`
                            # From `test/test_unit_tkts`
                            failure("Issue ticket migration failed!")
                            general(
                                f"Failed due to code '{issurslt[0]}' and reason '{issurslt[1]}' "
                                f"in {issurslt[2]} second(s)"
                            )
                else:  # pragma: no cover
                    # Tested already in `test_unit_iterpage`
                    # From `test/test_unit_tkts`
                    failure("Issue ticket information reading failed!")
                    general(
                        f"Failed due to code '{pagerslt[0]}' and reason '{pagerslt[1]}' in {pagerslt[2]} second(s)"  # noqa: E501
                    )
            success("Namespace assets transferring queue processed!")
            general(f"{standard.issutnfs} issue ticket(s) transferred")
            sys.exit(0)
        else:  # pragma: no cover
            # Tested already in `test_unit_getcount`
            # From `test/test_unit_tkts`
            failure("Source namespace issue ticket count failed!")
            general(
                f"Failed due to code '{qantrslt[0]}' and reason '{qantrslt[1]}' in {qantrslt[2]} second(s)"  # noqa: E501
            )
            sys.exit(1)
    else:
        for indx in standard.tktgroup:
            tkidrslt = moveobjc.iteriden(indx)
            section(f"Probing issue ticket #{indx}...")
            if tkidrslt[0] == 200:
                if not tkidrslt[1]:
                    general(f"Information retrieved in {tkidrslt[2]} second(s)")
                    callwait()
                    issurslt = moveobjc.itertkts(standard.issurslt)
                    section(
                        f"Migrating issue ticket {'with' if standard.movetags else 'without'} "
                        f"labels #{standard.issuiden} '{standard.issuname}' "
                        f"by '{standard.authname} (ID {standard.authorid})'..."
                    )
                    if issurslt[0] == 201:
                        general(f"Migrated to {issurslt[1]} in {issurslt[2]} second(s)")
                        if standard.movestat:
                            callwait()
                            section("Asserting issue ticket status...")
                            statrslt = moveobjc.iterstat()
                            if statrslt[0] == 200:
                                general(
                                    f"Asserted CLOSE status of the ticket in {statrslt[2]} second(s)"  # noqa: E501
                                )
                            elif statrslt[0] == 0:
                                general(
                                    "Assertion unnecessary due to the OPEN status of the ticket"
                                )
                            else:  # pragma: no cover
                                # Tested already in `test_unit_itertkts`
                                # From `test/test_unit_tkts`
                                failure("Issue ticket status assertion failed!")
                                general(
                                    f"Failed due to code '{statrslt[0]}' and reason '{statrslt[1]}' "  # noqa: E501
                                    f"      in {statrslt[2]} second(s)"
                                )
                        if standard.movecmts:
                            section("Reading comment information...")
                            standard.issucmts = standard.issurslt["comments"]
                            general(f"Found {len(standard.issucmts)} entities in 0.00 second(s)")
                            for kndx in standard.issucmts:
                                callwait()
                                cmtsrslt = moveobjc.itercmts(kndx)
                                section(
                                    f"Transferring comment (Entity {standard.cmtsqant} of {len(standard.issucmts)})..."  # noqa: E501
                                )
                                if cmtsrslt[0] == 201:
                                    general(
                                        f"Transferred to {cmtsrslt[1]} in {cmtsrslt[2]} second(s)"
                                    )
                                else:  # pragma: no cover
                                    # Tested already in `test_unit_itercmts`
                                    # From `test/test_unit_tkts`
                                    failure("Comment transfer failed!")
                                    general(
                                        f"Failed due to code '{cmtsrslt[0]}' and reason '{cmtsrslt[1]}'"  # noqa: E501
                                        f" in {cmtsrslt[2]} second(s)"
                                    )
                                    sys.exit(1)
                            standard.cmtsqant = 0
                    else:  # pragma: no cover
                        # Tested already in `test_unit_iteriden`
                        # From `test/test_unit_tkts`
                        failure("Issue ticket migration failed!")
                        general(
                            f"Failed due to code '{issurslt[0]}' and reason '{issurslt[1]}' in {issurslt[2]} second(s)"  # noqa: E501
                        )
                else:
                    general(
                        "Skipping issue ticket as the issue ticket status does not match the provided status"  # noqa: E501
                    )
            else:  # pragma: no cover
                # Tested already in `test_unit_getcount`
                # From `test/test_unit_tkts`
                failure("Issue ticket probing failed!")
                general(
                    f"Failed due to code '{tkidrslt[0]}' and reason '{tkidrslt[1]}' in {tkidrslt[2]} second(s)"  # noqa: E501
                )
        success("Namespace assets transferring queue processed!")
        general(f"{standard.issutnfs} issue ticket(s) transferred")
        sys.exit(0)
